@@ -721,9 +721,14 @@ class enigma_ui
         // get user's identities
         $identities = $this->rc->user->list_identities(null, true);
         $checkbox   = new html_checkbox(array('name' => 'identity[]'));
-        foreach ((array) $identities as $idx => $ident) {
+
+        $plugin     = $this->rc->plugins->exec_hook('enigma_user_identities', array('identities' => $identities));
+        $identities = $plugin['identities'];
+
+        foreach ($identities as $idx => $ident) {
             $name = empty($ident['name']) ? ($ident['email']) : $ident['ident'];
-            $identities[$idx] = html::label(null, $checkbox->show($name, array('value' => $name)) . rcube::Q($name));
+            $attr = array('value' => $idx, 'data-name' => $ident['name'], 'data-email' => $ident['email']);
+            $identities[$idx] = html::label(null, $checkbox->show($idx, $attr) . rcube::Q($name));
         }
 
         $table->add('title', html::label('key-name', rcube::Q($this->enigma->gettext('newkeyident'))));
@@ -919,7 +924,10 @@ class enigma_ui
             $attrib['id'] = 'enigma-message';
 
             if ($sig instanceof enigma_signature) {
-                $sender = ($sig->name ? $sig->name . ' ' : '') . '<' . $sig->email . '>';
+                $sender = $sig->name ?: '';
+                if ($sig->email) {
+                    $sender .= ' <' . $sig->email . '>';
+                }
 
                 if ($sig->valid === enigma_error::UNVERIFIED) {
                     $attrib['class'] = 'enigmawarning';
@@ -934,7 +942,13 @@ class enigma_ui
                 }
                 else {
                     $attrib['class'] = 'enigmawarning';
-                    $msg = rcube::Q(str_replace('$sender', $sender, $this->enigma->gettext('siginvalid')));
+                    if ($sender) {
+                        $msg = rcube::Q(str_replace('$sender', $sender, $this->enigma->gettext('siginvalid')));
+                    }
+                    else {
+                        $msg = rcube::Q(str_replace('$keyid', enigma_key::format_id($sig->id),
+                            $this->enigma->gettext('signokey')));
+                    }
                 }
             }
             else if ($sig && $sig->getCode() == enigma_error::KEYNOTFOUND) {
