@@ -1100,7 +1100,7 @@ class rcube_sieve_script
         }
 
         // multi-line string
-        if (preg_match('/[\r\n\0]/', $str) || strlen($str) > 1024) {
+        if (preg_match('/[\r\n\0]/', $str)) {
             return sprintf("text:\n%s\n.\n", self::escape_multiline_string($str));
         }
         // quoted-string
@@ -1277,11 +1277,16 @@ class rcube_sieve_script
                             $text    .= substr($str, $position, $pos - $position);
                             $position = $pos + 2;
 
-                            if ($str[$pos] == "\n"
-                                || ($str[$pos] == "\r" && $str[$pos + 1] == "\n")
-                            ) {
+                            if ($str[$position] == "\n") {
                                 break;
                             }
+
+                            if ($str[$position] == "\r" && $str[$position + 1] == "\n") {
+                                $position++;
+                                break;
+                            }
+
+                            $text .= "\n.";
                         }
 
                         // remove dot-stuffing
