@@ -1,6 +1,6 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
  | Copyright (C) 2005-2012, The Roundcube Dev Team                       |
@@ -26,7 +26,7 @@
  */
 class rcube_smtp
 {
-    private $conn = null;
+    private $conn;
     private $response;
     private $error;
     private $anonymize_log = 0;
@@ -47,7 +47,7 @@ class rcube_smtp
      *
      * @return bool  Returns true on success, or false on error
      */
-    public function connect($host=null, $port=null, $user=null, $pass=null)
+    public function connect($host = null, $port = null, $user = null, $pass = null)
     {
         $rcube = rcube::get_instance();
 
@@ -59,10 +59,10 @@ class rcube_smtp
 
         // let plugins alter smtp connection config
         $CONFIG = $rcube->plugins->exec_hook('smtp_connect', array(
-            'smtp_server'    => $host ? $host : $rcube->config->get('smtp_server'),
-            'smtp_port'      => $port ? $port : $rcube->config->get('smtp_port', 25),
-            'smtp_user'      => $user ? $user : $rcube->config->get('smtp_user'),
-            'smtp_pass'      => $pass ? $pass : $rcube->config->get('smtp_pass'),
+            'smtp_server'    => $host ?: $rcube->config->get('smtp_server'),
+            'smtp_port'      => $port ?: $rcube->config->get('smtp_port', 25),
+            'smtp_user'      => $user !== null ? $user : $rcube->config->get('smtp_user'),
+            'smtp_pass'      => $pass !== null ? $pass : $rcube->config->get('smtp_pass'),
             'smtp_auth_cid'  => $rcube->config->get('smtp_auth_cid'),
             'smtp_auth_pw'   => $rcube->config->get('smtp_auth_pw'),
             'smtp_auth_type' => $rcube->config->get('smtp_auth_type'),
@@ -95,6 +95,9 @@ class rcube_smtp
             $use_tls   = true;
         }
 
+        // Handle per-host socket options
+        rcube_utils::parse_socket_options($CONFIG['smtp_conn_options'], $smtp_host);
+
         if (!empty($CONFIG['smtp_helo_host'])) {
             $helo_host = $CONFIG['smtp_helo_host'];
         }
@@ -145,7 +148,7 @@ class rcube_smtp
 
         $smtp_user = str_replace('%u', $rcube->get_user_name(), $CONFIG['smtp_user']);
         $smtp_pass = str_replace('%p', $rcube->get_user_password(), $CONFIG['smtp_pass']);
-        $smtp_auth_type = empty($CONFIG['smtp_auth_type']) ? NULL : $CONFIG['smtp_auth_type'];
+        $smtp_auth_type = $CONFIG['smtp_auth_type'] ?: null;
 
         if (!empty($CONFIG['smtp_auth_cid'])) {
             $smtp_authz = $smtp_user;
@@ -183,7 +186,7 @@ class rcube_smtp
      *
      * @param string Sender e-Mail address
      *
-     * @param mixed  Either a comma-seperated list of recipients
+     * @param mixed  Either a comma-separated list of recipients
      *               (RFC822 compliant), or an array of recipients,
      *               each RFC822 valid. This may contain recipients not
      *               specified in the headers, for Bcc:, resending
@@ -336,7 +339,6 @@ class rcube_smtp
         }
     }
 
-
     /**
      * This is our own debug handler for the SMTP connection
      */
@@ -446,7 +448,7 @@ class rcube_smtp
      * bare addresses (forward paths) that can be passed to sendmail
      * or an smtp server with the rcpt to: command.
      *
-     * @param mixed Either a comma-seperated list of recipients
+     * @param mixed Either a comma-separated list of recipients
      *              (RFC822 compliant), or an array of recipients,
      *              each RFC822 valid.
      *
